/*
 * protocol support for handle packet with innova,
 *
 * 0.0.1:		tcp/icmp/udp support
 */

#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>

#include <stdlib.h>

#include "innova.h"

/*
 * function for single protocol checksum computing, all referred to 
 *
 * unsigned short checksum(const unsigned short *, int len);
 * on utilf.c
 */
void inline fix_ip_checksum(register struct innova_packet *pkt)
	{
	unsigned short cmptlen =0x00;
	unsigned int sum;

	/* initialize shared buffer in parser.c */
	append_in_buffer(NULL, 0x00);

	pkt->ip->check =0x00;

	/* 
	 * I don't use ip->ihl << 2 because shoud be that the ip
	 * options is on another pointer and not after iphpGader 
	 */
	cmptlen +=append_in_buffer(pkt->ip, sizeof(struct iphdr));

	if(pkt->ip_opt !=NULL)
		cmptlen +=append_in_buffer(pkt->ip_opt, pkt->ip_opt_len);

	sum =half_cksum((unsigned short *)get_b_addr(), cmptlen);
	pkt->ip->check =compute_sum(sum);
	}

void inline fix_tcp_checksum(register struct innova_packet *pkt)
	{
	unsigned short cmptlen =0x00;
	unsigned int sum;

	pkt->tcp->check =0x00;

	append_in_buffer(NULL, 0x00);

	/* tcp header, must be present */
	cmptlen +=append_in_buffer(pkt->tcp, sizeof(struct tcphdr));

	/* tco options and application data, should not be present */
	if(pkt->tcp_opt !=NULL)
		cmptlen +=append_in_buffer(pkt->tcp_opt, pkt->tcp_opt_len);
	if(pkt->data !=NULL)
		cmptlen +=append_in_buffer(pkt->data, pkt->data_len);

	sum =half_cksum((unsigned short *)&pkt->ip->saddr, 8);
	sum +=htons(IPPROTO_TCP + sizeof(struct tcphdr) + pkt->tcp_opt_len + pkt->data_len);
	sum +=half_cksum((unsigned short *)get_b_addr(), cmptlen);

	pkt->tcp->check = compute_sum(sum);
	}

/* udp and icmp checksum work how is working ip checksum ... */
void inline fix_udp_checksum(register struct innova_packet *pkt)
	{
	unsigned short cmptlen =0x00;
	unsigned int sum;

	append_in_buffer(NULL, 0x00);

	pkt->udp->check =0x00;

	cmptlen +=append_in_buffer(pkt->udp, sizeof(struct udphdr));

	if(pkt->data !=NULL)
		cmptlen +=append_in_buffer(pkt->data, pkt->data_len);

	sum =half_cksum((unsigned short *)get_b_addr(), cmptlen);
	pkt->udp->check =compute_sum(sum);

	}

void inline fix_icmp_checksum(register struct innova_packet *pkt)
	{
	unsigned short cmptlen =0x00;
	unsigned int sum;

	append_in_buffer(NULL, 0x00);

	pkt->icmp->checksum =0x00;

	cmptlen +=append_in_buffer(pkt->icmp, sizeof(struct icmphdr));

	if(pkt->data !=NULL)
		cmptlen +=append_in_buffer(pkt->data, pkt->data_len);

	sum =half_cksum((unsigned short *)get_b_addr(), cmptlen);
	pkt->icmp->checksum =compute_sum(sum);
	}

int is_supported(char *proto_name)
	{
	int proto_val =atoi(proto_name);

	switch(proto_val)
		{
		case IPPROTO_IP:
		case IPPROTO_TCP:
		case IPPROTO_UDP:
		case IPPROTO_ICMP:
		case IPPROTO_IPIP:
		case IPPROTO_GRE:
		case IPPROTO_PIM:
		case IPPROTO_RSVP:
			return proto_val;
		default:
			return 0x00;
		}

	/* gcc is our best friend */
	return 123;
	}

inline int verify_ip_check(struct iphdr *ip)
{
	unsigned short check, swap_cksum =ip->check;
	unsigned int sum;

	/*
	 * ip check is computed with check set to 0
	 */
	ip->check =0x00;

	sum =half_cksum((unsigned short *)ip, ip->ihl *4);

	check =compute_sum(sum);

	ip->check =swap_cksum;

	return (check ==swap_cksum);
	
}

inline void
fill_ip_options(struct innova_packet *t, const unsigned char *buf, int *len)
	{
	char optlen;

	*len =(t->ip->ihl *4);

	optlen =(*len -sizeof(struct iphdr));

	t->ip_opt =(void *)(buf +sizeof(struct iphdr));
	t->ip_opt_len =optlen;

	}

inline void 
fill_tcphdr(struct innova_packet *t, const unsigned char *buf, int *len)
	{
	t->tcp =(struct tcphdr *)&buf[t->ip->ihl *4];

	*len =(t->tcp->doff *4);

	if(*len > sizeof(struct tcphdr))
		{
		unsigned char optlen =(*len -sizeof(struct tcphdr));

		t->tcp_opt =(void *)((unsigned char *)t->tcp + *len -optlen);
		t->tcp_opt_len =optlen;
		}
	}
	
inline void 
fill_icmphdr(struct innova_packet *t, const unsigned char *buf, int *len)
	{
	t->icmp =(struct icmphdr *)&buf[t->ip->ihl *4];
	*len =sizeof(struct icmphdr);
	}

inline void 
fill_udphdr(struct innova_packet *t, const unsigned char *buf, int *len)
	{
	t->udp =(struct udphdr *)&buf[t->ip->ihl *4];
	*len =sizeof(struct udphdr);
	}
