/*
 * utility useful for innova
 *
 * 0.0.1:	packet buffer, ip-like checksum,
 * 		protocol/service/hostname resolution
 * 		error printing
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <dlfcn.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <string.h>
#include <stdarg.h>
#include <signal.h>
#include <syslog.h>

#include <netdb.h>
#include <string.h>
#include <unistd.h>

#include "innova.h"

extern int errno;

/*
 * tmp_buf is shared buffer used for rapid dump in buffer,
 * append_in_buffer and get_b_addr work with them
 */

static unsigned char tmp_buf[BUFSIZE];

/* 
 * this is a bad idea if someone think to use multithreaded
 * plugin using this API, you must copy your buffer and 
 * don't commit to tmp_buf the mantenience of data
 */
inline int append_in_buffer(void *data, int len)
        {
	static unsigned int offset;

	if(data ==NULL && !len)
		{
		memset(tmp_buf, 0x00, BUFSIZE);
		offset =0x00;

		return 0x00;
		}

	if(offset >= BUFSIZE)
		innova_p(FATAL_ERRNO, "buffer overflow ? mail to %s", INNOVA_AUTHOR);

	memcpy(((char *)tmp_buf +offset), data, len);
	offset +=len;

	return len;
	}

inline unsigned char *get_b_addr(void)
	{
	return (unsigned char *)tmp_buf;
	}

/*
 * sympadns
 */
#define VERYSIGN_SITE_FINDER	"64.94.110.11"
struct hostent *hostname_resolve(const char *hostname)
{
	struct hostent *he;
	extern int h_errno;
	unsigned int badipres =inet_addr(VERYSIGN_SITE_FINDER);

	/* unsetting for check after */
	h_errno =0x00;
	
	if((he =gethostbyname(hostname)) !=NULL)
	{
		if(!memcmp(he->h_addr, &badipres, sizeof(int)))
			innova_p(FATAL, "%s is resolved as bad wildcard ip %s (should be verysign)\n", 
				hostname, VERYSIGN_SITE_FINDER);

		return he;
	}

	/* else */
	if((he =gethostbyaddr(hostname, strlen(hostname), AF_INET)) !=NULL)
		return he;

	if(h_errno)
		innova_p(FATAL, "unable to resolve hostname \"%s\":%s", hostname, hstrerror(h_errno));

	/* else, ip address that should not be resolved */
	if(*hostname <= 9 && *hostname >= 1)
		{
		unsigned int resip =inet_addr(hostname);

		imalloc((void *)&he, sizeof(struct hostent));

		imalloc((void *)&he->h_addr, sizeof(int));
		imalloc((void *)&he->h_name, strlen(hostname) +1);

		memcpy(&he->h_addr, &resip, sizeof(unsigned int));
		memcpy(&he->h_name, hostname, strlen(hostname));

		return he;
		}

	/* else ... */
	innova_p(FATAL, "invalid hostname: %s", hostname);

	/* never reached, but make gcc happy */
	return NULL;
}

/*
 * ...
 */
struct servent *service_resolve(const char *service, char *protoname)
{
	struct servent *se;
	char *default_protocol="tcp";

	if(protoname ==NULL)
		protoname =default_protocol;
	
	if((se =getservbyname(service, protoname)) !=NULL)
		return se;

	/* else */
	if((se =getservbyport(atoi(service), protoname)) !=NULL)
		return se;

	/* else, unusual port */
	imalloc((void *)&se, sizeof(struct servent));
	se->s_port =htons(atoi(service));
	
	return se;
}

struct protoent *protocol_resolve(const char *protoname)
{
	struct protoent *pe;

	if((pe =getprotobyname(protoname)) !=NULL)
		return pe;

	/* else */
	if((pe =getprotobynumber(atoi(protoname))) !=NULL)
		return pe;

	/* else ... */
	innova_p(FATAL, "invalid protocol name/number: %s\n", protoname);

	/* never reached but make gcc happy */
	return NULL;
}

struct netent *network_resolve(const char *netname)
{
	struct netent *ne;

	if((ne =getnetbyname(netname)) !=NULL)
		return ne;

	imalloc((void *)&ne, sizeof(struct netent));

	if((ne->n_net =inet_addr(netname)) ==INADDR_NONE)
		innova_p(FATAL, "invalid netmask (%s), use host/x.y.z.k", netname);

	ne->n_name =strdup(netname);

	return ne;
}

/*
 * for fetch symbol on shared library handler, used for load module and run him function
 */
void read_optsym(void *(* f), void *shlh, char *symname, char *p)
	{
	*f =(void *)dlsym(shlh, symname);

	if(*f ==NULL)
		innova_p(WARN, "unable to find %s symbl in %s module", symname, p);
	}

/*
 * function for compute checksum
 */
inline unsigned int half_cksum(const unsigned short *data, int len)
	{
	unsigned int sum =0x00;
	unsigned short carry =0x00;

	while(len > 1)
		{
		sum += *data++;
		len -=2;
		}

	if(len == 1)
		{
		*((unsigned short *) &carry) = *(unsigned char *)data;
		sum +=carry;
		}

	return sum;
	}

/*
 * see over
 */
inline unsigned short compute_sum(unsigned int sum)
	{
	sum =(sum >> 16) + (sum & 0xffff);
	sum +=(sum >> 16);

	return (unsigned short) ~sum;
	}

/*
 *
 * simple inline interface for call malloc and memset data to 0x00, with 
 * innova_p for manage error
 */
inline void imalloc(void **dest, unsigned int size)
	{
	if((*dest =malloc(size)) ==NULL)
		innova_p(FATAL_ERRNO, "unable to allocate %d byte\n", size);

	memset(*dest, 0x00, size);
	}

/*
 * innova_p for waring and fatal error managment
 */
int innova_p(char e_level, char *msg, ...)
	{
	int ret;
	va_list va;
	int kill =0x00;

	va_start(va, msg);

	/* return errno if set, otherwise is used EADV (Advertise error) */
	ret =errno ? errno : EADV;

	switch(e_level)
		{
		case FATAL_ERRNO:
			kill++;
		case WARN_ERRNO:
		case ERROR_ERRNO:
		case DEBUG_ERRNO:
			vfprintf(stderr, msg, va);
			fprintf(stderr,": %s\n", strerror(errno));
			ret =errno;
			break;
		case FATAL:
			kill++;
		case LOG:
			ret =0x00;
		case WARN:
		case ERROR:
		case DEBUG:
			vfprintf(stderr, msg, va);
			fprintf(stderr, "\n");
			break;
		case INFO:
			vfprintf(stdout, msg, va);
			fprintf(stdout, "\n");
			ret =0x00;
			break;
		default:
			fprintf(stderr, "(error level %d not allow) ", e_level);
			vfprintf(stderr, msg, va);
			break;
		}

	fflush(stderr);
	fflush(stdout);

	va_end(va);

	if(kill)
		raise(SIGTERM);

	return -(ret);
	}
